@@ -29,7 +29,6 @@ class View(Widget):
     def __init__(self, layout: Layout = None, name: str | None = None) -> None:
         self.layout: Layout = layout or self.layout_factory()
         self.mouse_over: Widget | None = None
-        self.focused: Widget | None = None
         self.widgets: set[Widget] = set()
         self.named_widgets: dict[str, Widget] = {}
         self._mouse_style: Style = Style()
@@ -98,7 +97,7 @@ class View(Widget):
         if message.layout:
             await self.root_view.refresh_layout()
             self.log("LAYOUT")
-            # await self.app.refresh()
+
         display_update = self.root_view.layout.update_widget(self.console, widget)
         if display_update is not None:
             self.app.display(display_update)
@@ -209,7 +208,7 @@ class View(Widget):
             )
 
     async def forward_event(self, event: events.Event) -> None:
-
+        event.set_forwarded()
         if isinstance(event, (events.Enter, events.Leave)):
             await self.post_message(event)
 
@@ -237,12 +236,12 @@ class View(Widget):
                 widget, _region = self.get_widget_at(event.x, event.y)
             except NoWidget:
                 return
-            scroll_widget = widget or self.focused
+            scroll_widget = widget
             if scroll_widget is not None:
                 await scroll_widget.forward_event(event)
         else:
-            if self.focused is not None:
-                await self.focused.forward_event(event)
+            self.log("view.forwarded", event)
+            await self.post_message(event)
 
     async def action_toggle(self, name: str) -> None:
         widget = self.named_widgets[name]
